package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.core.base.entity.BaseEnergyStorageDeviceEnt;
import pps.core.base.entity.BasePowerLineStorageEnt;
import pps.core.base.enums.BusinessError;
import pps.core.base.mapper.BaseEnergyStorageDeviceMapper;
import pps.core.base.mapper.BasePowerLineStorageMapper;
import pps.core.base.service.data.base_energy_storage_device.*;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.List;
import java.util.Objects;

/**
 * 储能设备模块
 *
 * @author ZWT
 * @date 2023/09/08
 */
@XService
public class BaseEnergyStorageDeviceService {

    @XText("新增")
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseEnergyStorageDevice(XContext context, CreateBaseEnergyStorageDeviceInput input) {
        return XTransactionHelper.begin(context, () -> {
            BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
            BaseEnergyStorageDeviceEnt entity = XCopyUtils.copyNewObject(input, BaseEnergyStorageDeviceEnt.class);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }

    @XText("更新")
    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseEnergyStorageDevice(XContext context, UpdateBaseEnergyStorageDeviceInput input) {
        return XTransactionHelper.begin(context, () -> {
            BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
            BaseEnergyStorageDeviceEnt entity = this.selectOneByDeviceId(input.getId(), mapper);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            XCopyUtils.copyObject(input, entity);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    @XText("删除")
    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseEnergyStorageDevice(XContext context, DeleteBaseEnergyStorageDeviceInput input) {
        String storageId = input.getId();
        if (this.checkStorageIsReference(context, storageId)) {
            return XServiceResult.error(context, BusinessError.StorageReference);
        }
        return XTransactionHelper.begin(context, () -> {
            BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
            BaseEnergyStorageDeviceEnt entity = this.selectOneByDeviceId(storageId, mapper);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            entity = new BaseEnergyStorageDeviceEnt();
            entity.setId(storageId);
            entity.setIsDeleted(BusinessConstant.ZERO);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    @XText("根据id获取详情")
    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseEnergyStorageDeviceOutput> getBaseEnergyStorageDevice(XContext context, GetBaseEnergyStorageDeviceInput input) {
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        BaseEnergyStorageDeviceEnt entity = this.selectOneByDeviceId(input.getId(), mapper);
        if (Objects.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseEnergyStorageDeviceOutput output = XCopyUtils.copyNewObject(entity, GetBaseEnergyStorageDeviceOutput.class);
        return XSingleResult.success(output);
    }

    /**
     * 分页查询
     * GET /base/base-energy-storage-device/query-base-energy-storage-device
     * 接口ID：105602201
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105602201
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryBaseEnergyStorageDeviceOutput}>
     */
    @XText("分页查询")
    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseEnergyStorageDeviceOutput> queryBaseEnergyStorageDevice(XContext context, QueryBaseEnergyStorageDeviceInput input) {
        String ouName = input.getOuName();
        String deviceName = input.getDeviceName();
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        QueryWrapper<BaseEnergyStorageDeviceEnt> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotEmpty(input.getLineId())) {
            queryWrapper.lambda()
                    .notExists("SELECT 1 FROM base_power_line_storage w WHERE base_energy_storage_device.id = w.storage_id AND w.is_deleted = 1 AND w.line_id <> {0}", input.getLineId())
                    .eq(BaseEnergyStorageDeviceEnt::getOuId, input.getOuId());
        } else {
            List<String> allListByOuId = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
            queryWrapper.lambda()
                    .in(CollUtil.isNotEmpty(allListByOuId), BaseEnergyStorageDeviceEnt::getOuId, allListByOuId);
        }
        queryWrapper.lambda()
                .like(StrUtil.isNotEmpty(ouName), BaseEnergyStorageDeviceEnt::getOuName, ouName)
                .like(StrUtil.isNotEmpty(deviceName), BaseEnergyStorageDeviceEnt::getDeviceName, deviceName)
                .eq(BaseEnergyStorageDeviceEnt::getIsDeleted, BusinessConstant.ONE)
                .orderByDesc(BaseEnergyStorageDeviceEnt::getModifyTime);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseEnergyStorageDeviceOutput.class);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 检查储能设备是否被引用
     *
     * @param context   上下文
     * @param storageId 储能设备ID
     * @return boolean
     */
    private boolean checkStorageIsReference(XContext context, String storageId) {
        BasePowerLineStorageMapper mapper = context.getBean(BasePowerLineStorageMapper.class);
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePowerLineStorageEnt>()
                .eq(BasePowerLineStorageEnt::getStorageId, storageId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
        );
        return count > 0;
    }

    /**
     * 通过设备ID查询设备信息
     *
     * @param deviceId 设备id
     * @param mapper   映射器
     * @return {@link BaseEnergyStorageDeviceEnt}
     */
    private BaseEnergyStorageDeviceEnt selectOneByDeviceId(String deviceId, BaseEnergyStorageDeviceMapper mapper) {
        return mapper.selectOne(new LambdaQueryWrapper<BaseEnergyStorageDeviceEnt>()
                .eq(BaseModel::getId, deviceId)
        );
    }
}