package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line.GetBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.GetBasePowerLineViewOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantInput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantOutput;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageInput;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageOutput;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageViewOutput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.core.base.entity.*;
import pps.core.base.mapper.*;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;

import java.util.Collection;
import java.util.List;

/**
 * 输电线路配置Cloud模块
 *
 * @author ZWT
 * @date 2023/09/05 15:37
 */
@XService
public class BasePowerLineCloudServiceImpl implements IBasePowerLineCloudService {

    /**
     * 输电线路配置Cloud模块--通过ID查询线路基本信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBasePowerLineViewOutput}>
     */
    @Override
    public XSingleResult<GetBasePowerLineViewOutput> getBasePowerLineById(XContext context, GetBasePowerLineInput input) {
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        BasePowerLineEnt entity = mapper.selectOne(new LambdaQueryWrapper<BasePowerLineEnt>()
                .eq(BaseModel::getId, input.getId())
        );
        return XSingleResult.success(XCopyUtils.copyNewObject(entity, GetBasePowerLineViewOutput.class));
    }

    /**
     * 输电线路配置Cloud模块--条件查询线路基本信息列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineOutput}>
     */
    @Override
    public XListResult<DynamicQueryBasePowerLineOutput> getBasePowerLineList(XContext context, DynamicQueryBasePowerLineInput input) {
        String lineId = input.getLineId();
        List<String> lineIds = input.getLineIds();
        String ouId = input.getOuId();
        String upperLineId = input.getUpperLineId();
        String lineName = input.getLineName();
        String gridTypeKey = input.getGridTypeKey();
        String lineTypeKey = input.getLineTypeKey();
        String strategyId = input.getStrategyId();
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        List<BasePowerLineEnt> selectList = mapper.selectList(new LambdaQueryWrapper<BasePowerLineEnt>()
                .select(BaseModel::getId, BasePowerLineEnt::getLineName)
                .in(CollUtil.isNotEmpty(lineIds), BaseModel::getId, lineIds)
                .eq(StringUtils.isNotBlank(lineId), BaseModel::getId, lineId)
                .eq(StringUtils.isNotBlank(ouId), BasePowerLineEnt::getOuId, ouId)
                .eq(StringUtils.isNotBlank(upperLineId), BasePowerLineEnt::getUpperLineId, upperLineId)
                .like(StringUtils.isNotBlank(lineName), BasePowerLineEnt::getLineName, lineName)
                .eq(StringUtils.isNotBlank(gridTypeKey), BasePowerLineEnt::getGridTypeKey, gridTypeKey)
                .eq(StringUtils.isNotBlank(lineTypeKey), BasePowerLineEnt::getLineTypeKey, lineTypeKey)
                .eq(StringUtils.isNotBlank(strategyId), BasePowerLineEnt::getStrategyId, strategyId)
                .orderByDesc(BaseModel::getModifyTime)
        );
        return XListResult.success(XCopyUtils.copyNewList(selectList, DynamicQueryBasePowerLineOutput.class));
    }

    /**
     * 输电线路配置Cloud模块--条件查询线路关联井口信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineWellheadViewOutput}>
     */
    @Override
    public XListResult<DynamicQueryBasePowerLineWellheadViewOutput> getPowerLineWellheadList(XContext context, DynamicQueryBasePowerLineWellheadInput input) {
        BasePowerLineWellheadViewMapper wellheadViewMapper = context.getBean(BasePowerLineWellheadViewMapper.class);
        List<BasePowerLineWellheadView> wellheadViews = wellheadViewMapper.selectWellheadListByParam(
                BasePowerLineWellheadView.builder()
                        .lineId(input.getLineId())
                        .wellheadIds(input.getWellheadIds())
                        .build()
        );
        return XListResult.success(XCopyUtils.copyNewList(wellheadViews, DynamicQueryBasePowerLineWellheadViewOutput.class));
    }

    /**
     * 输电线路配置Cloud模块--条件查询线路关联电站信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLinePlantOutput}>
     */
    @Override
    public XListResult<DynamicQueryBasePowerLinePlantOutput> queryPowerLinePlantListByParam(XContext context, DynamicQueryBasePowerLinePlantInput input) {
        String plantId = input.getPlantId();
        Collection<String> lineIds = input.getLineIds();
        BasePowerLinePlantMapper mapper = context.getBean(BasePowerLinePlantMapper.class);
        List<BasePowerLinePlantEnt> list = mapper.selectList(
                new LambdaQueryWrapper<BasePowerLinePlantEnt>()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(StringUtils.isNotBlank(plantId), BasePowerLinePlantEnt::getPlantId, plantId)
                        .in(CollUtil.isNotEmpty(lineIds), BasePowerLinePlantEnt::getLineId, lineIds)
        );
        return XListResult.success(XCopyUtils.copyNewList(
                list,
                DynamicQueryBasePowerLinePlantOutput.class
        ));
    }

    /**
     * 输电线路配置Cloud模块--条件查询线路关联储能设备参数平均值
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineStorageViewOutput}>
     */
    @Override
    public XListResult<DynamicQueryBasePowerLineStorageViewOutput> queryPowerLineStorageAvgByLineIds(XContext context, DynamicQueryBasePowerLineStorageInput input) {
        BasePowerLineStorageViewMapper mapper = context.getBean(BasePowerLineStorageViewMapper.class);
        List<BasePowerLineStorageView> list = mapper.selectStorageAverageByLineIds(input.getLineIds());
        return XListResult.success(XCopyUtils.copyNewList(
                list,
                DynamicQueryBasePowerLineStorageViewOutput.class
        ));
    }

    /**
     * 输电线路配置Cloud模块--条件查询线路关联储能设备信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineStorageOutput}>
     */
    @Override
    public XListResult<DynamicQueryBasePowerLineStorageOutput> queryPowerLineStorageListByLineIds(XContext context, DynamicQueryBasePowerLineStorageInput input) {
        BasePowerLineStorageMapper storageMapper = context.getBean(BasePowerLineStorageMapper.class);
        List<BasePowerLineStorageEnt> list = storageMapper.selectList(new LambdaQueryWrapper<BasePowerLineStorageEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .in(BasePowerLineStorageEnt::getLineId, input.getLineIds())
        );
        return XListResult.success(XCopyUtils.copyNewList(
                list,
                DynamicQueryBasePowerLineStorageOutput.class
        ));
    }
}