package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.space.entity.*;
import pps.core.space.mapper.*;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.service.data.space_optimize_mid_period.GetSpaceOptimizeMidPeriodViewInput;
import pps.core.space.service.data.space_optimize_mid_period.GetSpaceOptimizeMidPeriodViewOutput;
import pps.core.space.service.data.space_optimize_mid_period.QuerySpaceOptimizeMidPeriodViewInput;
import pps.core.space.service.data.space_optimize_mid_period.QuerySpaceOptimizeMidPeriodViewOutput;
import pps.core.space.service.data.space_optimize_mid_wellhead.GetSpaceOptimizeMidWellheadOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 中短期间开优化模块
 *
 * @author ZWT
 * @date 2023/09/07
 */
@XService
@XText("中短期间开优化模块")
public class SpaceOptimizeMidPeriodService extends SpaceOptimizeBaseService {

    /**
     * 中短期间开优化--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QuerySpaceOptimizeMidPeriodViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("中短期间开优化--分页列表")
    public XPageResult<QuerySpaceOptimizeMidPeriodViewOutput> querySpaceOptimizeMidPeriodView(XContext context, QuerySpaceOptimizeMidPeriodViewInput input) {
        SpaceOptimizeMidPeriodViewMapper mapper = context.getBean(SpaceOptimizeMidPeriodViewMapper.class);
        SpaceOptimizeMidPeriodView record = XCopyUtils.copyNewObject(input, SpaceOptimizeMidPeriodView.class);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SpaceOptimizeMidPeriodView> list = mapper.selectList(record);
        PageInfo<SpaceOptimizeMidPeriodView> pageInfo = new PageInfo<>(list);
        List<QuerySpaceOptimizeMidPeriodViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceOptimizeMidPeriodViewOutput.class);
        if (CollUtil.isNotEmpty(outputs)) {
            Map<String, String> gridTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.GRID_TYPE);
            Map<String, String> ouMap = ServiceUtil.getOuMapByOuIds(context, outputs.stream()
                    .map(QuerySpaceOptimizeMidPeriodViewOutput::getOuId)
                    .distinct()
                    .collect(Collectors.toList())
            );
            String lineName = ServiceUtil.getPowerLineNameById(context, input.getLineId());
            outputs.forEach(output -> {
                output.setLineName(lineName);
                output.setOuName(ouMap.get(output.getOuId()));
                output.setGridTypeName(gridTypeMap.get(output.getGridTypeKey()));
                output.setOptimizeStateValue(output.getOptimizeState().equals(0) ? BusinessConstant.OPTIMIZED_C : BusinessConstant.NO_OPTIMIZATION_C);
            });
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 中短期间开优化--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSpaceOptimizeMidPeriodViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("中短期间开优化--详情")
    public XSingleResult<GetSpaceOptimizeMidPeriodViewOutput> getSpaceOptimizeMidPeriodView(XContext context, GetSpaceOptimizeMidPeriodViewInput input) {
        SpaceOptimizeMidPeriodMapper mapper = context.getBean(SpaceOptimizeMidPeriodMapper.class);
        String midPeriodId = input.getId();
        SpaceOptimizeMidPeriodEnt period = mapper.selectById(midPeriodId);
        if (Objects.isNull(period)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSpaceOptimizeMidPeriodViewOutput output = XCopyUtils.copyNewObject(period, GetSpaceOptimizeMidPeriodViewOutput.class);
        output.setDateList(ServiceUtil.getOptimizeDateList(period.getCreateTime(), period.getOptimizeDeadline()));
        //查间开制度详情
        SpaceInstitutionDetailMapper detailMapper = context.getBean(SpaceInstitutionDetailMapper.class);
        SpaceInstitutionDetailEnt detailEnt = detailMapper.selectById(period.getInstitutionId());
        if (StringUtils.equals(String.valueOf(BusinessConstant.ONE), Optional.ofNullable(detailEnt.getGridTypeKey())
                .orElse(String.valueOf(BusinessConstant.ZERO)))) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(period.getOptimizeDeadline());
            output.setStrategyDetailList(ServiceUtil.getStrategyDetailList(context,
                    GetBasePriceStrategyDetailInput.builder()
                            .lineId(period.getLineId())
                            .strategyMonth(String.valueOf(calendar.get(Calendar.MONTH) + 1))
                            .build()));
        } else {
            output.setStrategyDetailList(new ArrayList<>(0));
        }
        output.setGridTypeKey(detailEnt.getGridTypeKey());
        //查井口列表
        SpaceOptimizeMidWellheadMapper wellheadMapper = context.getBean(SpaceOptimizeMidWellheadMapper.class);
        List<SpaceOptimizeMidWellheadEnt> wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeMidWellheadEnt>()
                .eq(SpaceOptimizeMidWellheadEnt::getMidPeriodId, midPeriodId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
        );
        if (CollUtil.isNotEmpty(wellheadList)) {
            List<GetSpaceOptimizeMidWellheadOutput> wellheadOutputs = XCopyUtils.copyNewList(wellheadList, GetSpaceOptimizeMidWellheadOutput.class);
            //查井口配置列表
            SpaceOptimizeMidDurationMapper durationMapper = context.getBean(SpaceOptimizeMidDurationMapper.class);
            List<SpaceOptimizeMidDurationEnt> durationList = durationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeMidDurationEnt>()
                    .eq(SpaceOptimizeMidDurationEnt::getMidPeriodId, midPeriodId)
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .orderByAsc(SpaceOptimizeMidDurationEnt::getIsOptimize, SpaceOptimizeMidDurationEnt::getOpenWellTime)
            );
            if (CollUtil.isNotEmpty(durationList)) {
                List<SpaceOptimizeDateDuration> durationOutputs = XCopyUtils.copyNewList(durationList, SpaceOptimizeDateDuration.class);
                Map<String, List<SpaceOptimizeDateDuration>> collect = new HashMap<>(16);
                for (SpaceOptimizeDateDuration durationOutput : durationOutputs) {
                    if (StringUtils.isEmpty(durationOutput.getGenerationTypeKey())) {
                        durationOutput.setGenerationTypeKey(BusinessConstant.PHOTOVOLTAIC);
                    }
                    List<SpaceOptimizeDateDuration> tmpList = collect.get(durationOutput.getWellheadId());
                    if (CollUtil.isEmpty(tmpList)) {
                        tmpList = new ArrayList<>(16);
                        tmpList.add(durationOutput);
                        collect.put(durationOutput.getWellheadId(), tmpList);
                    } else {
                        tmpList.add(durationOutput);
                    }
                }
                wellheadOutputs.forEach(w -> {
                    if (collect.containsKey(w.getWellheadId())) {
                        Map<Integer, List<SpaceOptimizeDateDuration>> listMap = collect.get(w.getWellheadId()).stream()
                                .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getIsOptimize));
                        w.setOptimizeList(ServiceUtil.getAllDateDuration(listMap.get(0)));
                        w.setInitializeList(ServiceUtil.getAllDateDuration(listMap.get(1)));
                    } else {
                        w.setOptimizeList(new ArrayList<>(0));
                        w.setInitializeList(new ArrayList<>(0));
                    }
                });
            }
            output.setWellheadList(wellheadOutputs);
        } else {
            output.setWellheadList(new ArrayList<>(0));
        }
        //查功率预测信息
        output.setPowerGenerationList(
                super.getAveragePowerGenerationHourList(context, DynamicQueryPlantPredictedPowerInput.builder()
                        .plantIds(super.getOptimizeLineRelation(context, CollUtil.newArrayList(detailEnt))
                                .getPlantIdsByLineIdMap()
                                .get(detailEnt.getLineId()))
                        .startTime(DateUtil.date(period.getCreateTime())
                                .toString(BusinessConstant.DATE_FORMAT_DAY))
                        .endTime(DateUtil.date(period.getOptimizeDeadline())
                                .toString(BusinessConstant.DATE_FORMAT_DAY))
                        .dateType(BusinessConstant.ONE)
                        .build())
        );
        return XSingleResult.success(output);
    }
}