package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudInput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.prediction.entity.PhotovoltaicElectricityConsumptionEnt;
import pps.core.prediction.mapper.PhotovoltaicElectricityConsumptionMapper;
import pps.core.prediction.service.data.energy_consumption_analysis.GetEnergyConsumptionAnalysisInput;
import pps.core.prediction.service.data.energy_consumption_analysis.GetEnergyConsumptionAnalysisOutput;
import pps.core.prediction.service.data.energy_consumption_analysis.GetEnergyConsumptionAnalysisViewOutput;
import pps.core.prediction.service.data.energy_consumption_curve.GetEnergyConsumptionCurveOutput;
import pps.core.prediction.service.data.photovoltaic_electricity_consumption.QueryPhotovoltaicElectricityConsumptionInput;
import pps.core.prediction.service.data.photovoltaic_electricity_consumption.QueryPhotovoltaicElectricityConsumptionOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 能耗分析模块(测试用)
 * todo: 模拟测试用,后续替换
 *
 * @author ZWT
 * @date 2023/09/27 15:57
 */
@XText("能耗分析模块")
@XService
public class EnergyConsumptionAnalysisService extends StatisticAnalysisBaseService {

    /**
     * 能耗分析--绿电消纳
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetEnergyConsumptionCurveOutput}>
     */
    @XText("能耗分析--绿电消纳")
    @XApiAnonymous
    @XApiGet
    public XListResult<GetEnergyConsumptionAnalysisOutput> queryGreenElectricityAbsorption(XContext context, GetEnergyConsumptionAnalysisInput input) {
        //查包含当前组织的所有子组织列表
        List<String> orgIdsByPath = super.getOrgIdsByPath(context, input.getOuId());
        //查组织下的光伏电站
        List<GetBasePhotovoltaicPlantCloudOutput> photovoltaicPlantList = super.getPhotovoltaicPlantByParam(context,
                GetBasePhotovoltaicPlantCloudInput.builder()
                        .ouIds(orgIdsByPath)
                        .build()
        );
        List powerGenerationList = null;
        if (CollUtil.isNotEmpty(photovoltaicPlantList)) {
            powerGenerationList = super.getPhotovoltaicPowerGeneration(context, input.getDateType(),
                    photovoltaicPlantList.stream()
                            .map(GetBasePhotovoltaicPlantCloudOutput::getId)
                            .collect(Collectors.toList()),
                    input.getStartTime(), input.getEndTime()
            );
        }
        List<GetEnergyConsumptionAnalysisOutput> outputList = new ArrayList<>(0);
        if (CollUtil.isNotEmpty(powerGenerationList)) {
            XCopyUtils.copyList(powerGenerationList, outputList, GetEnergyConsumptionAnalysisOutput.class);
            //todo 设置一个值计算用电量
            BigDecimal flag = new BigDecimal("0.8");
            BigDecimal power;
            for (GetEnergyConsumptionAnalysisOutput output : outputList) {
                power = output.getPower();
                if (Objects.isNull(power) ||
                        power.compareTo(BigDecimal.ZERO) <= BusinessConstant.ZERO
                ) {
                    output.setElectricityConsumption(BigDecimal.ZERO);
                    continue;
                }
                //计算负荷
                output.setElectricityConsumption(
                        power.multiply(flag)
                                .setScale(2, BigDecimal.ROUND_HALF_UP)
                );
            }
        }
        return XListResult.success(outputList);
    }

    /**
     * 能耗分析--电量使用趋势/数据概览
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetEnergyConsumptionAnalysisViewOutput}>
     */
    @XText("能耗分析--电量使用趋势/数据概览")
    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetEnergyConsumptionAnalysisViewOutput> queryElectricityUsageTrend(XContext context, GetEnergyConsumptionAnalysisInput input) {
        //查包含当前组织的所有子组织列表
        List<String> orgIdsByPath = super.getOrgIdsByPath(context, input.getOuId());
        //查组织下的光伏电站
        List<GetBasePhotovoltaicPlantCloudOutput> photovoltaicPlantList = super.getPhotovoltaicPlantByParam(context,
                GetBasePhotovoltaicPlantCloudInput.builder()
                        .ouIds(orgIdsByPath)
                        .build()
        );
        List powerGenerationList = null;
        if (CollUtil.isNotEmpty(photovoltaicPlantList)) {
            powerGenerationList = super.getPhotovoltaicPowerGeneration(context, input.getDateType(),
                    photovoltaicPlantList.stream()
                            .map(GetBasePhotovoltaicPlantCloudOutput::getId)
                            .collect(Collectors.toList()),
                    input.getStartTime(), input.getEndTime()
            );
        }
        List<GetEnergyConsumptionAnalysisOutput> powerList = new ArrayList<>(0);
        BigDecimal greenElectricity = BigDecimal.ZERO;
        BigDecimal cityElectricity = BigDecimal.ZERO;
        if (CollUtil.isNotEmpty(powerGenerationList)) {
            XCopyUtils.copyList(powerGenerationList, powerList, GetEnergyConsumptionAnalysisOutput.class);
            //todo 设置一个值计算用电量
            BigDecimal flag = new BigDecimal("1.5");
            BigDecimal power;
            for (GetEnergyConsumptionAnalysisOutput output : powerList) {
                power = output.getPower();
                if (Objects.isNull(power) ||
                        power.compareTo(BigDecimal.ZERO) <= BusinessConstant.ZERO
                ) {
                    output.setElectricityConsumption(BigDecimal.ZERO);
                    continue;
                }
                BigDecimal bigDecimal = power.multiply(flag)
                        .setScale(2, BigDecimal.ROUND_HALF_UP);
                //计算负荷
                output.setElectricityConsumption(bigDecimal);
                //计算市电/绿电总和
                greenElectricity = greenElectricity.add(power);
                cityElectricity = cityElectricity.add(bigDecimal);
            }
        }
        return XSingleResult.success(
                GetEnergyConsumptionAnalysisViewOutput.builder()
                        .cityElectricity(cityElectricity)
                        .greenElectricity(greenElectricity)
                        .powerList(powerList)
                        .build()
        );
    }

    /**
     * 能耗分析--用电排行
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryPhotovoltaicElectricityConsumptionOutput}>
     */
    @XText("能耗分析--用电排行")
    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryPhotovoltaicElectricityConsumptionOutput> electricityConsumptionRanking(XContext context, QueryPhotovoltaicElectricityConsumptionInput input) {
        //查包含当前组织的所有子组织列表
        List<String> orgIdsByPath = super.getOrgIdsByPath(context, input.getOuId());
        PhotovoltaicElectricityConsumptionMapper mapper = context.getBean(PhotovoltaicElectricityConsumptionMapper.class);
        PageHelper.startPage(input.getPage(), input.getLimit());
        LambdaQueryWrapper<PhotovoltaicElectricityConsumptionEnt> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(CollUtil.isNotEmpty(orgIdsByPath), PhotovoltaicElectricityConsumptionEnt::getOuId, orgIdsByPath);
        //取当前时间
        DateTime date = DateUtil.beginOfDay(DateUtil.date());
        switch (input.getDateType()) {
            case 0:
                queryWrapper.eq(PhotovoltaicElectricityConsumptionEnt::getDataDate, date);
                break;
            case 1:
                queryWrapper.between(PhotovoltaicElectricityConsumptionEnt::getDataDate, date, DateUtil.offsetDay(date, 7));
                break;
            case 2:
                queryWrapper.between(PhotovoltaicElectricityConsumptionEnt::getDataDate, date, DateUtil.offsetDay(date, 30));
                break;
            case 3:
                queryWrapper.between(PhotovoltaicElectricityConsumptionEnt::getDataDate, input.getStartTime(), input.getEndTime());
                break;
            default:
        }
        queryWrapper.orderByDesc(PhotovoltaicElectricityConsumptionEnt::getTotalElectricity);
        List<PhotovoltaicElectricityConsumptionEnt> list = mapper.selectList(queryWrapper);
        PageInfo<PhotovoltaicElectricityConsumptionEnt> pageInfo = new PageInfo<>(list);
        List<QueryPhotovoltaicElectricityConsumptionOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryPhotovoltaicElectricityConsumptionOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /*-----------------------------------private-----------------------------------*/
}
