package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.prediction.service.IPlantPredictedPowerCloudService;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerOutput;
import pps.core.prediction.entity.PlantPredictedPowerDataEnt;
import pps.core.prediction.entity.PlantPredictedPowerLongTermDataEnt;
import pps.core.prediction.mapper.PlantPredictedPowerDataMapper;
import pps.core.prediction.mapper.PlantPredictedPowerLongTermDataMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;

import java.util.ArrayList;
import java.util.List;

/**
 * 光伏预测Cloud模块
 *
 * @author ZWT
 * @date 2023/09/13 19:05
 */
@XService
public class PlantPredictedPowerCloudServiceImpl implements IPlantPredictedPowerCloudService {

    /**
     * 光伏预测Cloud模块--条件查询时段发电量平均值
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    @Override
    public XListResult<DynamicQueryPlantPredictedPowerOutput> queryAveragePowerGenerationListByParam(XContext context, DynamicQueryPlantPredictedPowerInput input) {
        String plantId = input.getPlantId();
        List<String> plantIds = input.getPlantIds();
        String yearTime = input.getYearTime();
        String monthTime = input.getMonthTime();
        String startTime = input.getStartTime();
        String endTime = input.getEndTime();
        Integer dateType = input.getDateType();
        List list = null;
        switch (dateType) {
            case 0:
                PlantPredictedPowerLongTermDataMapper longTermDataMapper = context.getBean(PlantPredictedPowerLongTermDataMapper.class);
                list = longTermDataMapper.selectList(
                        new QueryWrapper<PlantPredictedPowerLongTermDataEnt>()
                                .select("year_time",
                                        "month_time",
                                        "hour_time",
                                        "min_time",
                                        "IFNULL( AVG( power ), 0 ) AS power",
                                        "STR_TO_DATE( CONCAT( hour_time, ':', min_time ), '%H:%i:%s' ) AS create_time")
                                .lambda()
                                .eq(StringUtils.isNotBlank(plantId), PlantPredictedPowerLongTermDataEnt::getPlantId, plantId)
                                .in(CollUtil.isNotEmpty(plantIds), PlantPredictedPowerLongTermDataEnt::getPlantId, plantIds)
                                .eq(StringUtils.isNotBlank(yearTime), PlantPredictedPowerLongTermDataEnt::getYearTime, yearTime)
                                .eq(StringUtils.isNotBlank(monthTime), PlantPredictedPowerLongTermDataEnt::getMonthTime, monthTime)
                                .between(!StringUtils.isAnyBlank(startTime, endTime), PlantPredictedPowerLongTermDataEnt::getDataDate, startTime, endTime)
                                .groupBy(PlantPredictedPowerLongTermDataEnt::getYearTime,
                                        PlantPredictedPowerLongTermDataEnt::getMonthTime,
                                        PlantPredictedPowerLongTermDataEnt::getHourTime,
                                        PlantPredictedPowerLongTermDataEnt::getMinTime)
                                .orderByAsc(PlantPredictedPowerLongTermDataEnt::getHourTime)
                );
                break;
            case 1:
                PlantPredictedPowerDataMapper mapper = context.getBean(PlantPredictedPowerDataMapper.class);
                list = mapper.selectList(
                        new QueryWrapper<PlantPredictedPowerDataEnt>()
                                .select("year_time",
                                        "month_time",
                                        "hour_time",
                                        "min_time",
                                        "IFNULL( AVG( power ), 0 ) AS power",
                                        "STR_TO_DATE( CONCAT( hour_time, ':', min_time ), '%H:%i:%s' ) AS create_time")
                                .lambda()
                                .eq(StringUtils.isNotBlank(plantId), PlantPredictedPowerDataEnt::getPlantId, plantId)
                                .in(CollUtil.isNotEmpty(plantIds), PlantPredictedPowerDataEnt::getPlantId, plantIds)
                                .eq(StringUtils.isNotBlank(yearTime), PlantPredictedPowerDataEnt::getYearTime, yearTime)
                                .eq(StringUtils.isNotBlank(monthTime), PlantPredictedPowerDataEnt::getMonthTime, monthTime)
                                .between(!StringUtils.isAnyBlank(startTime, endTime), PlantPredictedPowerDataEnt::getDataDate, startTime, endTime)
                                .groupBy(PlantPredictedPowerDataEnt::getYearTime,
                                        PlantPredictedPowerDataEnt::getMonthTime,
                                        PlantPredictedPowerDataEnt::getHourTime,
                                        PlantPredictedPowerDataEnt::getMinTime)
                                .orderByAsc(PlantPredictedPowerDataEnt::getHourTime)
                );
                break;
            default:
        }
        List<DynamicQueryPlantPredictedPowerOutput> outputs;
        if (CollUtil.isEmpty(list)) {
            outputs = new ArrayList<>(0);
        } else {
            outputs = XCopyUtils.copyNewList(list, DynamicQueryPlantPredictedPowerOutput.class);
        }
        return XListResult.success(outputs);
    }

    /**
     * 光伏预测Cloud模块--获取每小时平均发电量列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    @Override
    public XListResult<DynamicQueryPlantPredictedPowerOutput> queryAveragePowerGenerationHourListByParam(XContext context, DynamicQueryPlantPredictedPowerInput input) {
        String plantId = input.getPlantId();
        List<String> plantIds = input.getPlantIds();
        String yearTime = input.getYearTime();
        String monthTime = input.getMonthTime();
        String startTime = input.getStartTime();
        String endTime = input.getEndTime();
        Integer dateType = input.getDateType();
        List list = null;
        switch (dateType) {
            case 0:
                PlantPredictedPowerLongTermDataMapper longTermDataMapper = context.getBean(PlantPredictedPowerLongTermDataMapper.class);
                list = longTermDataMapper.selectList(
                        new QueryWrapper<PlantPredictedPowerLongTermDataEnt>()
                                .select("hour_time",
                                        "IFNULL( AVG( power ), 0 ) AS power")
                                .lambda()
                                .eq(StringUtils.isNotBlank(plantId), PlantPredictedPowerLongTermDataEnt::getPlantId, plantId)
                                .in(CollUtil.isNotEmpty(plantIds), PlantPredictedPowerLongTermDataEnt::getPlantId, plantIds)
                                .eq(StringUtils.isNotBlank(yearTime), PlantPredictedPowerLongTermDataEnt::getYearTime, yearTime)
                                .eq(StringUtils.isNotBlank(monthTime), PlantPredictedPowerLongTermDataEnt::getMonthTime, monthTime)
                                .between(!StringUtils.isAnyBlank(startTime, endTime), PlantPredictedPowerLongTermDataEnt::getDataDate, startTime, endTime)
                                .groupBy(PlantPredictedPowerLongTermDataEnt::getHourTime)
                                .orderByAsc(PlantPredictedPowerLongTermDataEnt::getHourTime)
                );
                break;
            case 1:
                PlantPredictedPowerDataMapper mapper = context.getBean(PlantPredictedPowerDataMapper.class);
                list = mapper.selectList(
                        new QueryWrapper<PlantPredictedPowerDataEnt>()
                                .select("hour_time",
                                        "IFNULL( AVG( power ), 0 ) AS power")
                                .lambda()
                                .eq(StringUtils.isNotBlank(plantId), PlantPredictedPowerDataEnt::getPlantId, plantId)
                                .in(CollUtil.isNotEmpty(plantIds), PlantPredictedPowerDataEnt::getPlantId, plantIds)
                                .eq(StringUtils.isNotBlank(yearTime), PlantPredictedPowerDataEnt::getYearTime, yearTime)
                                .eq(StringUtils.isNotBlank(monthTime), PlantPredictedPowerDataEnt::getMonthTime, monthTime)
                                .between(!StringUtils.isAnyBlank(startTime, endTime), PlantPredictedPowerDataEnt::getDataDate, startTime, endTime)
                                .groupBy(PlantPredictedPowerDataEnt::getHourTime)
                                .orderByAsc(PlantPredictedPowerDataEnt::getHourTime)
                );
                break;
            default:
        }
        List<DynamicQueryPlantPredictedPowerOutput> outputs;
        if (CollUtil.isEmpty(list)) {
            outputs = new ArrayList<>(0);
        } else {
            outputs = XCopyUtils.copyNewList(list, DynamicQueryPlantPredictedPowerOutput.class);
        }
        return XListResult.success(outputs);
    }
}
